﻿/*jslint browser: true, regexp: true, plusplus: true */
/*global XIMSSSession: false, GibberishAES: false, basePathRef: false, $: false, require: false, useSMSAuth: false */
var sessionInstance,
	storedUserName,
	storedPassword,
	storedSid,
	storedLogInMethod,
	storedLogoutURL,
	storedSessionXML = null,
	requireLoading = false,
	requireCallback,
	userLang = __stringsLang;
(function () {
	"use strict";
	var parser = /^(?:([^:\/?#]+):)?(?:\/\/((?:(([^:@]*):?([^:@]*))?@)?([^:\/?#]*)(?::(\d*))?))?((((?:[^?#\/]*\/)*)([^?#]*))(?:\?([^#]*))?(?:#(.*))?)/, //less intuitive, more accurate to the specs
		querystring_parser = /(?:^|&|;)([^&=;]*)=?([^&;]*)/g, // supports both ampersand and semicolon-delimted query string key/value pairs
		fragment_parser = /(?:^|&|;)([^&=;]*)=?([^&;]*)/g, // supports both ampersand and semicolon-delimted fragment key/value pairs
		key = [ "source", "protocol", "authority", "userInfo", "user", "password",
			"host", "port", "relative", "path", "directory", "file", "query",
			"fragment" ], // keys available to query
		parseUri = function (url) {
			var str = decodeURI(url), res = parser.exec(str), uri = {
				attr : {},
				param : {},
				seg : {}
			},
				i = 14;

			while (i--) {
				uri.attr[key[i]] = res[i] || "";
			}

			// build query and fragment parameters
			uri.param.query = {};
			uri.param.fragment = {};

			uri.attr.query.replace(querystring_parser, function ($0, $1, $2) {
				if ($1) {
					uri.param.query[$1] = $2;
				}
			});

			uri.attr.fragment.replace(fragment_parser, function ($0, $1, $2) {
				if ($1) {
					uri.param.fragment[$1] = $2;
				}
			});

			// split path and fragement into segments
			uri.seg.path = uri.attr.path.replace(/^\/+|\/+$/g, '').split('/');
			uri.seg.fragment = uri.attr.fragment.replace(/^\/+|\/+$/g, '').split('/');

			// compile a 'base' domain attribute
			uri.attr.base = uri.attr.host ? uri.attr.protocol + "://" + uri.attr.host +
					(uri.attr.port ? ":" + uri.attr.port : '') : '';

			return uri;
		},
		url = window.location.toString().replace(/^(.*?\?)((.*?&)*?)((p-)?)(username=[^&]*?)@/gi, "$1$2$4$6%40"),
		data = parseUri(url),
		host = data.attr.host,
		loginInput = $(".pronto-login__login"),
		passInput = $(".pronto-login__password"),
		smsInput = $(".pronto-login__sms"),
		setLocalItem = function (name, value) {
			try {
				localStorage.setItem(name, value);
			} catch (e) {}
		},
		getLocalItem = function (name) {
			try {
				var value = localStorage.getItem(name);
				return value;
			} catch (e) {
				return null;
			}
		},
		removeLocalItem = function (name) {
			try {
				var value = localStorage.removeItem(name);
			} catch (e) {}
		},
		asyncPreserver = function (xml) {
			// Just hack to override setUnknownAsyncProcessor handler
			if (typeof this.realAsyncProcessor !== "undefined" && this.realAsyncProcessor !== null) {
				this.realAsyncProcessor(xml);
			}
			if (!sessionInstance.preservedAsync) {
				sessionInstance.preservedAsync = [xml];
			} else {
				sessionInstance.preservedAsync.push(xml);
			}
		},
		getErrorString = function (str) {
			if (typeof __stringsXml === "undefined") {
				return str;
			}
			var labelId = "ErrorCodes." + str,
				query = labelId.split(".").map(function (part) { return '[name="' + part + '"]'; }).join(" "),
				string = __stringsXml.querySelector(query);
			if (!string) {
				return str;
			}
			if (string.nodeName === "string") {
				return string.textContent;
			}
			return string;			
		},
		handleError = function (error, sms) {
			if (sms) {
				$(".pronto-login-sms__start").show();
			} else {
				$("#pronto-login__start").show();
			}
			$(".pronto-login__siginingin").hide();
			if (error === "server communication error") {
				error = "cannot connect to the server";
			}
			$(".pronto-login__error-common").show().text(getErrorString(error));
		},
		normalizeUsername = function (uName) {
			if (uName.indexOf("@") > -1) {
				return uName;
			}
			return uName + "@" + host;
		},
		loginWithSms = function (uName, password) {
			$.ajax({
				url: data.attr.base + "/cgi-bin/smscheck1.pl",
				type: "POST",
				data: {
					username: normalizeUsername(uName),
					password: password
				},
				dataType: "text",
				success: function (result) {
					if (!result) {
						handleError("unknown error");
					}
					if ($.trim(result) === "OK") {
						$(".pronto-login-sms__start").show();
						$(".pronto-login__siginingin").hide();
						$(".pronto-login__form").addClass("pronto-login-sms__form");
						smsInput.val("");
					} else if (result.indexOf("Error:") === 0) {
						handleError(result.substr("Error:".length));
					}
				},
				error: function () {
					handleError("server communication error");
				}
			});
		},
		loadRequire = function (showError, successCallback) {
			if (successCallback) {
				requireCallback = successCallback;
			}
			if (requireLoading) {
				return;
			}
			requireLoading = true;
			$.ajax({
				url: basePathRef + "require.js?rand=" + __revisionNumber,
				dataType: "script",
				cache: true,
				success: function () {
					requireLoading = false;
					if (requireCallback) {
						requireCallback();
					}
				},
				error: function (jqxhr, settings, exception) {
					requireLoading = false;
					if (showError) {
						$(".pronto-login__error-common").show().text(getErrorString(exception));
					}
				}
			});
		},
		loadMain = function () {
			if (typeof require === "undefined") {
				loadRequire(true, loadMain);
				return;
			}
			$.ajax({
				url: basePathRef + "main.production.js?rand=" + __revisionNumber,
				dataFilter: function (data, type) {
					return data + "\n\n//@ sourceURL=main.production.js\n\n//# sourceURL=main.production.js";
				},
				dataType: "script",
				cache: true,
				error: function (jqxhr, settings, exception) {
					$(".pronto-login__error-common").show().text(getErrorString(exception));
				}
			});
		},
		logIn = function (uName, passwordOrSid, isSid) {
			$(".pronto-login__start").hide();
			$(".pronto-login__siginingin").show();
			$(".pronto-login__error").hide();

			if (!isSid && (useSMSAuth.toLowerCase() === "yes" || useSMSAuth.toLowerCase() === "true")) {
				loginWithSms(uName, passwordOrSid);
				return;
			}

			var params = {
					secureMode : data.attr.protocol.toLowerCase() === "https" ? "YES" : "NO",
					serverName : data.attr.host + (data.attr.port ? ":" + data.attr.port : ""),
					binding : "HTTP",
					userName : uName,
					loginMethod : isSid ? "sessionid" : "auto",
					sessionid : passwordOrSid,
					password : passwordOrSid,
					asyncMode : "syncPOST"
				},
				s = new XIMSSSession(params, null, function (session, error) {
					if (!error) {
						session.start();
						sessionInstance = session;
						session.setUnknownAsyncProcessor(session, asyncPreserver);
						storedPassword = isSid ? null : passwordOrSid;
						storedSid = isSid ? passwordOrSid : null;
						storedLogInMethod = isSid ? "sessionid" : "auto";
						var prefsReadNode = session.createXMLNode("prefsRead"), name = session.createXMLNode("name");
						name.appendChild(session.createTextNode("Language"));
						prefsReadNode.appendChild(name);
						session.sendRequest(prefsReadNode, null, function (responseXML, requestXML) {
							var langNode;
							if (responseXML.nodeName === "prefs") {
								langNode = responseXML.getElementsByTagName("Language")[0];
								userLang = (langNode && langNode.childNodes && langNode.childNodes.length > 0 ?
										langNode.childNodes[0].nodeValue : "") || "";
							}
						}, function (error, requestXML) {
							loadMain();
						}, true);
					} else {
						handleError(error);
						if (session && session.close) {
							session.close();
						}
					}
				});
			s.setAsyncProcessor(this, function (sessionXML) {
				var sessionXMLjQ = $(sessionXML);
				storedUserName = sessionXMLjQ.attr("userName");
				storedSessionXML = sessionXML;
			}, "session");
			return s;
		},
		loginWithSmsConfirm = function (uName, smsPassword) {
			$(".pronto-login__start").hide();
			$(".pronto-login__siginingin").show();
			$(".pronto-login__error").hide();
			$.ajax({
				url: data.attr.base + "/cgi-bin/smscheck2.pl",
				type: "POST",
				data: {
					username: normalizeUsername(uName),
					code: smsPassword
				},
				dataType: "text",
				success: function (result) {
					if (!result) {
						handleError("unknown error", true);
					} else if (result.indexOf("Error:") === 0) {
						handleError(result.substr("Error:".length), true);
					} else if (result.indexOf("Authorized:") === 0) {
						logIn(uName, result.split("\n")[0].split("\r")[0].split("Authorized: ")[1], true);
					}
				},
				error: function () {
					handleError("server communication error", true);
				}
			});
		},
		getPassEnc = function (password) {
			return GibberishAES.enc(password, host);
		},
		getLogInEnc = function (logIn) {
			return GibberishAES.enc(logIn, host);
		},
		decPass = function (passwordEnc) {
			return GibberishAES.dec(passwordEnc, host);
		},
		decLogIn = function (logInEnc) {
			return GibberishAES.dec(logInEnc, host);
		},
		fillCredentials = function () {
			var passEnc = getLocalItem("p"),
				logInEnc = getLocalItem("l"),
				pass = passEnc ? decPass(passEnc) : "",
				logIn = logInEnc ? decLogIn(logInEnc) : "";
			loginInput.val(logIn);
			passInput.val(pass);
		},
		fillStorage = function () {
			var username = loginInput.val(),
				password = passInput.val(),
				logInEnc = getLogInEnc(username),
				passEnc = getPassEnc(password);
			setLocalItem("p", passEnc);
			setLocalItem("l", logInEnc);
		},
		clearStorage = function () {
			removeLocalItem("p");
			removeLocalItem("l");
		},
		oldPass = "",
		oldLogIn = "",
		oldSmsCode = "",
		submitButton = $(".pronto-login__submit"),
		checkButton = function () {
			var username = loginInput.val(),
				password = passInput.val(),
				smsCode = smsInput.val();
			if (oldPass !== password || oldLogIn !== username || oldSmsCode !== smsCode) {
				if ((username && username.length > 0 && password && password.length > 0) || smsCode.length > 0) {
					submitButton.removeAttr("disabled");
				} else {
					submitButton.attr("disabled", "disabled");
				}
				if (!smsCode) {
					if ($(".pronto-login__remember-field")[0].checked) {
						fillStorage();
					} else {
						clearStorage();
					}
				}
				oldPass = password;
				oldLogIn = username;
				oldSmsCode = smsCode;
			}
		},
		checkLocalStorage = function () {
			var remember = getLocalItem("remember"),
				rememberValue = typeof remember === "string" ? (remember === "true") : remember,
				checkbox = $(".pronto-login__remember-field")[0];
			checkbox.checked = rememberValue;
			if (rememberValue) {
				fillCredentials();
			} else {
				clearStorage();
			}
		},
		recoveryVisibility = function (type) {
			$(".pronto-login__error-password-recovery").hide();
			$(".pronto-login__soccess-password-recovery").hide();
			if (type === "hide") {
				$(".pronto-login-password-recovery__start").hide();
				$(".pronto-login-auth__start").show();
			} else if (type === "show") {
				$(".pronto-login-auth__start").hide();
				$(".pronto-login-password-recovery__start").show();
				$(".pronto-login__recover-password").prop('disabled', false);
			}
		},
		recoverPassword = function (callback) {
			var xmlhttp = new XMLHttpRequest(),
				url = "/ximssLogin/",
				request = "<XIMSS><recoverPassword id='0' domain='",
				userName = "";

			xmlhttp.open("POST", url, true);
			xmlhttp.setRequestHeader('Content-Type', 'text/xml');
			xmlhttp.onreadystatechange = function () {
				if (xmlhttp.readyState === 4) {
					callback(xmlhttp.responseText);
				}
			};
			if ($(".pronto-login__recover-account-password").val()) {
				userName = $(".pronto-login__recover-account-password").val();
			}
			request += host + "' userName='";
			request += userName + "' /></XIMSS>";
			xmlhttp.send(request);
		},
		start = function () {
			var uname, sid, pwd;
			if (data.param.query.hasOwnProperty("logoutURL")) {
				storedLogoutURL = decodeURIComponent(data.param.query.logoutURL);
			}
			loadRequire();
			if ((data.param.query.hasOwnProperty("p-username") || data.param.query.hasOwnProperty("username")) &&
					(data.param.query.hasOwnProperty("p-sid") ||
					data.param.query.hasOwnProperty("sid") ||
					data.param.query.hasOwnProperty("password") ||
					data.param.query.hasOwnProperty("p-password"))) {
				uname = data.param.query.hasOwnProperty("p-username") ?
						decodeURIComponent(data.param.query["p-username"]) :
							decodeURIComponent(data.param.query.username);
				if (data.param.query.hasOwnProperty("p-sid") || data.param.query.hasOwnProperty("sid")) {
					sid = data.param.query.hasOwnProperty("p-sid") ?
							decodeURIComponent(data.param.query["p-sid"]) :
								decodeURIComponent(data.param.query.sid);
					logIn(uname, sid, true);
				} else {
					pwd = data.param.query.hasOwnProperty("p-password") ?
							decodeURIComponent(data.param.query["p-password"]) :
								decodeURIComponent(data.param.query.password);
					logIn(uname, pwd);
				}
			} else {
				$("#pronto-login__start").show();
				checkLocalStorage();
				checkButton();
				setInterval(checkButton, 100);
				$(".pronto-login__remember-field").change(function () {
					setLocalItem("remember", this.checked);
					if (this.checked) {
						fillStorage();
					} else {
						clearStorage();
					}
				});

				var forgotPasswordOption = $(".pronto-login__loginoptions-forgotpassword").text();
				if (forgotPasswordOption != "YES") {
					$(".pronto-login__recover-password-invite").hide();
				}

				$(".pronto-login__recover-password-invite").click(function () {
					var originalUserName = $(".pronto-login__login").val();
					recoveryVisibility("show");
					if (originalUserName) {
						$(".pronto-login__recover-account-password").val(originalUserName);
					}

				});
				$(".pronto-login__cancel-recover-password").click(function () {
					recoveryVisibility("hide");
				});
				$(".pronto-login__recover-password").click(function () {
					recoveryVisibility("hideMessages");
					$(".pronto-login__recover-password").prop('disabled', true);
					recoverPassword(function n(response) {
						var resp = $(response).find('response'),
							error = resp.attr('errorText');

						if (!resp.length) {
							error = "Wrong request";
						}

						if (error) {
							$(".pronto-login__error-password-recovery").show();
							$(".pronto-login__error-password-recovery").text(error);
							$(".pronto-login__recover-password").prop('disabled', false);
						} else {
							$(".pronto-login__error-password-recovery").hide();
							$(".pronto-login__soccess-password-recovery").show();
							setTimeout(function () {recoveryVisibility("hide"); }, 2000);
						}
					});
				});

				$("#mainFrom").submit(function () {
					if ($(".pronto-login-sms__start:visible").length > 0) {
						loginWithSmsConfirm(loginInput.val(), $("#smsPassword").val());
					} else {
						var username = loginInput.val(),
							password = passInput.val();
						// set fixed width to avoid width "jump" between state switches. I was unable to implement it with transitions
						$(".pronto-login__form").width($(".pronto-login__form").width());
						logIn(username, password);
					}
					return false;
				});
			}
		}

		$("#try_anyway_link").click(function () {
			$(".pronto-login").show();
			$("#pronto-login__not-supported-browser").hide();
			start();
		});

	start();


}());
